==> idp-plugin-oidc-op-impl-3.2.0-sources.jar: net/shibboleth/idp/plugin/oidc/op/profile/logic/DefaultTokenRevocationLifetimeLookupStrategy.java <==
/*
 * Licensed to the University Corporation for Advanced Internet Development,
 * Inc. (UCAID) under one or more contributor license agreements.  See the
 * NOTICE file distributed with this work for additional information regarding
 * copyright ownership. The UCAID licenses this file to You under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.shibboleth.idp.plugin.oidc.op.profile.logic;

import java.time.Duration;
import java.time.Instant;
import java.util.Date;
import java.util.function.Function;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.nimbusds.jwt.JWTClaimsSet;

/**
 * Default lookup function for fetching the token revocation lifetime from the given claims set. If an expiration
 * time is found from the claims set, a difference between now and it is returned. If the expiration time is in the
 * past, a {@link Duration#ZERO} is returned. If no expiration time is found, null is returned.
 */
public class DefaultTokenRevocationLifetimeLookupStrategy implements Function<JWTClaimsSet, Duration> {

    /** Class logger. */
    @Nonnull
    private final Logger log = LoggerFactory.getLogger(DefaultTokenRevocationLifetimeLookupStrategy.class);

    /** {@inheritDoc} */
    @Override
    @Nullable
    public Duration apply(@Nullable final JWTClaimsSet claimsSet) {
        if (claimsSet == null) {
            log.error("The given claims set was null, returning null");
            return null;
        }
        final Date expiration = claimsSet.getExpirationTime();
        if (expiration == null) {
            log.debug("No token expiration time found from the claims set, returning null");
            return null;
        }
        final Instant now = Instant.now();
        final Instant exp = expiration.toInstant();
        if (now.isAfter(exp)) {
            log.debug("Token expiration time was in the past, returning ZERO");
            return Duration.ZERO;
        }
        return Duration.between(now, exp);
    }
    
}

==> idp-plugin-oidc-op-impl-3.2.1-sources.jar: net/shibboleth/idp/plugin/oidc/op/profile/logic/DefaultTokenRevocationLifetimeLookupStrategy.java <==
/*
 * Licensed to the University Corporation for Advanced Internet Development,
 * Inc. (UCAID) under one or more contributor license agreements.  See the
 * NOTICE file distributed with this work for additional information regarding
 * copyright ownership. The UCAID licenses this file to You under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.shibboleth.idp.plugin.oidc.op.profile.logic;

import java.time.Duration;
import java.time.Instant;
import java.util.Date;
import java.util.function.Function;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.nimbusds.jwt.JWTClaimsSet;

import net.shibboleth.utilities.java.support.logic.Constraint;

/**
 * Default lookup function for fetching the token revocation lifetime from the given claims set. If an expiration
 * time is found from the claims set, a difference between now and it is first calculated and then the configurable
 * clock skew value and additional 5 minutes are added to the value. The result is returned.
 * 
 * <p>If the expiration time is in the past with clock skew considered, a {@link Duration#ZERO} is returned.</p>
 * 
 * <p>If no expiration time is found, null is returned.</p>
 */
public class DefaultTokenRevocationLifetimeLookupStrategy implements Function<JWTClaimsSet, Duration> {

    /** Class logger. */
    @Nonnull
    private final Logger log = LoggerFactory.getLogger(DefaultTokenRevocationLifetimeLookupStrategy.class);

    /** Positive clock skew adjustment to consider when calculating revocation lifetime. */
    @Nonnull private Duration clockSkew;

    /**
     * Constructor.
     */
    public DefaultTokenRevocationLifetimeLookupStrategy() {
        clockSkew = Duration.ofMinutes(5);
    }

    /**
     * Set the clock skew.
     * 
     * @param skew clock skew to set
     */
    public void setClockSkew(@Nonnull final Duration skew) {
        clockSkew = Constraint.isNotNull(skew, "Clock skew cannot be null").abs();
    }

    /** {@inheritDoc} */
    @Override
    @Nullable
    public Duration apply(@Nullable final JWTClaimsSet claimsSet) {
        if (claimsSet == null) {
            log.error("The given claims set was null, returning null");
            return null;
        }
        final Date expiration = claimsSet.getExpirationTime();
        if (expiration == null) {
            log.debug("No token expiration time found from the claims set, returning null");
            return null;
        }
        final Instant now = Instant.now();
        final Instant exp = expiration.toInstant();
        if (now.isAfter(exp.plus(clockSkew))) {
            log.debug("Token expiration time was in the past, returning ZERO");
            return Duration.ZERO;
        }
        return Duration.between(now, exp).abs().plus(Duration.ofMinutes(5)).plus(clockSkew);
    }
    
}
